@@ -1,5 +1,5 @@
 /* ATK -  Accessibility Toolkit
- * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
+ * Copyright 2001 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,14 @@
 
 #include <glib-object.h>
 
+#ifdef G_OS_WIN32
+#define STRICT
+#include <windows.h>
+#undef STRICT
+#undef FOCUS_EVENT		/* <windows.h> pollutes the namespace
+				 * like a six hundred pound gorilla */
+#endif
+
 #include "atk.h"
 #include "atkmarshal.h"
 #include "atk-enum-types.h"
@@ -45,6 +53,7 @@ enum
   PROP_TABLE_ROW_DESCRIPTION,
   PROP_TABLE_ROW_HEADER,
   PROP_TABLE_SUMMARY,
+  PROP_TABLE_CAPTION_OBJECT,
   PROP_LAST         /* gobject convention */
 };
 
@@ -122,6 +131,7 @@ static const gchar* atk_object_name_property_table_column_header = "accessible-t
 static const gchar* atk_object_name_property_table_row_description = "accessible-table-row-description";
 static const gchar* atk_object_name_property_table_row_header = "accessible-table-row-header";
 static const gchar* atk_object_name_property_table_summary = "accessible-table-summary";
+static const gchar* atk_object_name_property_table_caption_object = "accessible-table-caption-object";
 
 GType
 atk_object_get_type (void)
@@ -252,7 +262,7 @@ atk_object_class_init (AtkObjectClass *klass)
                                    PROP_TABLE_CAPTION,
                                    g_param_spec_string (atk_object_name_property_table_caption,
                                                         "Accessible Table Caption",
-                                                        "Is used to notify that the table caption has changed ",
+                                                        "Is used to notify that the table caption has changed; this property should not be used. accessible-table-caption-object should be used instead",
                                                         NULL,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
@@ -290,10 +300,13 @@ atk_object_class_init (AtkObjectClass *klass)
                                                         "Is used to notify that the table summary has changed ",
                                                         ATK_TYPE_OBJECT,
                                                         G_PARAM_READWRITE));
-  /*
-   * The signal "children_changed" supports two details:
-   * "add" and "remove"
-   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_TABLE_CAPTION_OBJECT,
+                                   g_param_spec_object (atk_object_name_property_table_caption_object,
+                                                        "Accessible Table Caption Object",
+                                                        "Is used to notify that the table caption has changed ",
+                                                        ATK_TYPE_OBJECT,
+                                                        G_PARAM_READWRITE));
   atk_object_signals[CHILDREN_CHANGED] =
     g_signal_new ("children_changed",
 		  G_TYPE_FROM_CLASS (klass),
@@ -321,11 +334,6 @@ atk_object_class_init (AtkObjectClass *klass)
                   g_cclosure_marshal_VOID__POINTER,
                   G_TYPE_NONE, 1,
                   G_TYPE_POINTER);
-  /*
-   * The "state_change" signal supports details, one for each accessible 
-   * state type
-   * (see atkstate.c).
-   */
   atk_object_signals[STATE_CHANGE] =
     g_signal_new ("state_change",
                   G_TYPE_FROM_CLASS (klass),
@@ -759,7 +767,7 @@ atk_object_set_role (AtkObject *accessible,
           if (old_role != ATK_ROLE_UNKNOWN)
           /* Do not notify for initial role setting */
             g_object_notify (G_OBJECT (accessible), atk_object_name_property_role);
-   	}
+        }
     }
 }
 
@@ -895,6 +903,14 @@ atk_object_real_set_property (GObject      *object,
       if (ATK_IS_VALUE (accessible))
         atk_value_set_current_value (ATK_VALUE (accessible), value);
       break;
+    case PROP_TABLE_SUMMARY:
+      if (ATK_IS_TABLE (accessible))
+        atk_table_set_summary (ATK_TABLE (accessible), g_value_get_object (value));
+      break;
+    case PROP_TABLE_CAPTION_OBJECT:
+      if (ATK_IS_TABLE (accessible))
+        atk_table_set_caption (ATK_TABLE (accessible), g_value_get_object (value));
+      break;
     default:
       break;
     }
@@ -936,6 +952,14 @@ atk_object_real_get_property (GObject      *object,
       if (ATK_IS_VALUE (accessible))
         atk_value_get_current_value (ATK_VALUE (accessible), value);
       break;
+    case PROP_TABLE_SUMMARY:
+      if (ATK_IS_TABLE (accessible))
+        g_value_set_object (value, atk_table_get_summary (ATK_TABLE (accessible)));
+      break;
+    case PROP_TABLE_CAPTION_OBJECT:
+      if (ATK_IS_TABLE (accessible))
+        g_value_set_object (value, atk_table_get_caption (ATK_TABLE (accessible)));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1162,6 +1186,23 @@ atk_role_get_name (AtkRole role)
   return name;
 }
 
+#ifdef G_OS_WIN32
+
+#undef ATKLOCALEDIR
+
+#define ATKLOCALEDIR get_atk_locale_dir()
+
+G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name)
+
+static char *
+get_atk_locale_dir (void)
+{
+  return g_win32_get_package_installation_subdirectory
+    (GETTEXT_PACKAGE, dll_name, "lib/locale");
+}
+
+#endif
+
 /**
  * atk_role_get_localized_name:
  * @role: The #AtkRole whose localized name is required
@@ -1179,6 +1220,8 @@ atk_role_get_localized_name (AtkRole role)
 #ifdef ENABLE_NLS
   if (!gettext_initialized)
     {
+      gettext_initialized = TRUE;
+
       bindtextdomain (GETTEXT_PACKAGE, ATKLOCALEDIR);
 #ifdef HAVE_BIND_TEXTDOMAIN_CODESET
       bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
